@@ -1,7 +1,7 @@
 
 /*
     xskat - a card game for 1 to 3 players.
-    Copyright (C) 2000  Gunter Gerhardt
+    Copyright (C) 2004  Gunter Gerhardt
 
     This program is free software; you can redistribute it freely.
     Use it at your own risk; there is NO WARRANTY.
@@ -34,8 +34,6 @@ typedef struct {
   char *t[NUM_LANG];
 } tx_typ;
 
-extern tx_typ textarr[];
-
 enum {
   TX_NULL,TX_KARO,TX_HERZ,TX_PIK,TX_KREUZ,TX_GRAND,TX_RAMSCH,
   TX_AS,TX_ZEHN,TX_KOENIG,TX_DAME,TX_BUBE,TX_NEUN,TX_ACHT,TX_SIEBEN,
@@ -69,11 +67,14 @@ enum {
   TX_PROTOKOLL,TX_WEITER,TX_PROTOKOLL_T,
   TX_IM_SKAT_IST,
   TX_IM_SKAT_WAR,
+  TX_URSPRUENG_SKAT,
   TX_GEREIZT_BIS,
   TX_GEWO_MIT,
   TX_VERL_MIT,
   TX_SPIELLISTE,TX_SPIELLISTE_T,TX_XSKAT,
-  TX_COPYRIGHT,TX_EMAIL,TX_HOMEPAGES,TX_URL1,TX_URL2,
+  TX_COPYRIGHT,TX_URL,
+  TX_DOWNLOAD1,TX_DOWNLOAD2,
+  TX_ALT1,TX_ALT2,TX_ALT3,TX_ALT4,
   TX_SORTIEREN,TX_AUFWAERTS,TX_ABWAERTS,TX_ALTERNIEREND,TX_SEQUENTIELL,
   TX_NORMAL,
   TX_GEREIZT_BIS_L,TX_GESPIELT_WIRD,TX_LETZTER_STICH,
@@ -82,7 +83,8 @@ enum {
   TX_FR2,
   TX_FR3,
   TX_FR4,
-  TX_OPTIONEN,TX_COMPUTER_T,TX_STRATEGIE,TX_PASSE_K,TX_SAGE18,
+  TX_OPTIONEN,TX_COMPUTER_T,TX_SPIELSTAERKE,TX_MM,TX_M,TX_0,TX_P,TX_PP,
+  TX_NAME,
   TX_VORSCHLAEGE,TX_VARIANTEN,
   TX_RAMSCH_SPIELEN,TX_IMMER,
   TX_RAMSCH_VARIANTEN_T,
@@ -94,7 +96,7 @@ enum {
   TX_VH_SCHOB,TX_MH_SCHOB,TX_HH_SCHOB,
   TX_KONTRA_SAGEN,TX_MIT_KONTRA,TX_AB18,TX_KONTRA,TX_KONTRA_T,TX_RE,TX_VON,
   TX_WIEDER,TX_SPIEL_WIEDER,TX_MIT_DEN_KARTEN,TX_MIR,
-  TX_VORHAND_WECHSELT,
+  TX_VORHAND_WECHSELT,TX_START,TX_ZURUECK,
   TX_GESCHWINDIGKEIT,TX_NIMM_STICH_NACH,TX_SEKUNDEN,TX_MAUS_KLICK,
   TX_ABKUERZUNG,TX_FRAGEN,TX_NIE,
   TX_BOCK_RUNDEN,TX_UND_RAMSCH,TX_FORTSETZEN,
@@ -125,10 +127,24 @@ enum {
   TX_SPRACHE,TX_GERMAN,TX_ENGLISH,
   TX_ALTE_REGELN,
   TX_INTRO1,TX_INTRO2,
+  TX_MEHRSPIELER,
+  TX_LAN_T,
+  TX_EIGENER_TISCH,
+  TX_ANDERER_TISCH,
+  TX_SPIELER_2,
+  TX_SPIELER_3,
+  TX_RECHNER_IP,
+  TX_TISCH_OEFFNEN,
+  TX_TISCH,
+  TX_ZUGRIFF_1,
+  TX_ZUGRIFF_2,
+  TX_MAN_XHOST,
+  TX_PLATZ_NEHMEN,
+  TX_SCHLIESST_FENSTER,
   TX_IRC_T,
   TX_IRC_VERBINDUNG_HERSTELLEN,
   TX_IRC_KONFIGURATION_SIEHE,
-  TX_IRC_MAN_XSKAT,
+  TX_MAN_XSKAT,
   TX_VERSCHIEDENE_VERSIONEN,
   TX_README_IRC,
   TX_NICHT_HIER,
@@ -163,4 +179,9 @@ enum {
   TX_NUM_TX
 };
 
+EXTERN tx_typ textarr[TX_NUM_TX];
+EXTERN char spnames[4][2][NUM_LANG][10];
+EXTERN char conames[2][2][40];
+EXTERN char usrname[2][40];
+
 #endif /* TEXT_H */
