@@ -1,7 +1,7 @@
 
 /*
     xskat - a card game for 1 to 3 players.
-    Copyright (C) 2000  Gunter Gerhardt
+    Copyright (C) 2004  Gunter Gerhardt
 
     This program is free software; you can redistribute it freely.
     Use it at your own risk; there is NO WARRANTY.
@@ -53,6 +53,8 @@ typedef struct {
 
 EXTERN OBJECT *actdial[3];
 EXTERN OBJECT *resdial[3];
+EXTERN int inputdi[3],inputlen[3];
+EXTERN char *inputbuf[3];
 EXTERN int ktrsag,ktrnext,schenknext;
 EXTERN tx_typ stichstr[2][3][3],spielstr[2][3];
 
@@ -60,10 +62,11 @@ EXTERN OBJECT dihand[]
 #ifdef XDIAL_C
 =
 {
-  {4,0,0,19,6,OB_NONE,OB_NONE},
-  {OB_CENTERED|OB_UNDERLINED,1,1,17,1,TOB(TX_HAND_F),OB_NONE},
-  {OB_EXIT,2,4,6,1,TOB(TX_JA),OB_NONE},
-  {OB_EXIT,11,4,6,1,TOB(TX_NEIN),OB_NONE}
+  {5,0,0,23,7,OB_NONE,OB_NONE},
+  {OB_CENTERED|OB_UNDERLINED,1,1,21,1,TOB(TX_HAND_F),OB_NONE},
+  {OB_EXIT,2,3,8,1,TOB(TX_JA),OB_NONE},
+  {OB_EXIT,13,3,8,1,TOB(TX_NEIN),OB_NONE},
+  {OB_CENTERED,1,5,21,1,OB_NONE,OB_NONE}
 }
 #endif
 ;
@@ -389,17 +392,19 @@ EXTERN OBJECT diwieder[]
 #ifdef XDIAL_C
 =
 {
-  {11,0,0,37,10,OB_NONE,OB_NONE},
+  {13,0,0,37,12,OB_NONE,OB_NONE},
   {OB_CENTERED|OB_UNDERLINED,1,1,35,1,TOB(TX_SPIEL_WIEDER),OB_NONE},
   {OB_CENTERED,1,3,35,1,TOB(TX_MIT_DEN_KARTEN),OB_NONE},
   {OB_CENTERED,2,5,9,1,TOB(TX_COMPUTER),OB_NONE},
   {OB_CENTERED,26,5,9,1,TOB(TX_COMPUTER),OB_NONE},
-  {OB_EXIT,2,6,9,1,TOB(TX_LINKS),OB_NONE},
-  {OB_EXIT,14,6,9,1,TOB(TX_MIR),OB_NONE},
-  {OB_EXIT,26,6,9,1,TOB(TX_RECHTS),OB_NONE},
+  {OB_BUTTON,2,6,9,1,TOB(TX_LINKS),1},
+  {OB_BUTTON,14,6,9,1,TOB(TX_MIR),1},
+  {OB_BUTTON,26,6,9,1,TOB(TX_RECHTS),-2},
   {OB_NONE,2,8,18,1,TOB(TX_VORHAND_WECHSELT),OB_NONE},
   {OB_BUTTON,23,8,6,1,TOB(TX_JA),1},
-  {OB_BUTTON,29,8,6,1,TOB(TX_NEIN),-1}
+  {OB_BUTTON,29,8,6,1,TOB(TX_NEIN),-1},
+  {OB_EXIT,2,10,12,1,TOB(TX_ZURUECK),OB_NONE},
+  {OB_EXIT,23,10,12,1,TOB(TX_START),OB_NONE},
 }
 #endif
 ;
@@ -594,11 +599,11 @@ EXTERN OBJECT diliste[3][66]
 #endif
 ;
 
-EXTERN OBJECT dioptions[3][24]
+EXTERN OBJECT dioptions[3][25]
 #ifdef XDIAL_C
 =
 {{
-  {24,0,0,38,21,OB_NONE,OB_NONE},
+  {25,0,0,38,21,OB_NONE,OB_NONE},
   {OB_CENTERED|OB_UNDERLINED,1,1,36,1,TOB(TX_XSKAT),OB_NONE},
   {OB_EXIT,2,3,34,1,TOB(TX_COPYRIGHT),OB_NONE},
   {OB_NONE,2,5,35,1,TOB(TX_SORTIEREN),OB_NONE},
@@ -616,6 +621,7 @@ EXTERN OBJECT dioptions[3][24]
   {OB_NONE,5,14,32,1,OB_NONE,OB_NONE},
   {OB_NONE,5,15,32,1,OB_NONE,OB_NONE},
   {OB_EXIT,13,15,12,1,TOB(TX_SCHENKEN),OB_NONE},
+  {OB_EXIT,8,15,22,1,TOB(TX_MEHRSPIELER),OB_NONE},
   {OB_EXIT,5,17,13,1,TOB(TX_SPIELLISTE),OB_NONE},
   {OB_EXIT,20,17,13,1,TOB(TX_OPTIONEN),OB_NONE},
   {OB_EXIT,5,19,13,1,TOB(TX_ENDE),OB_NONE},
@@ -626,31 +632,49 @@ EXTERN OBJECT dioptions[3][24]
 #endif
 ;
 
-EXTERN OBJECT dicopyr[3][12]
+EXTERN OBJECT dicopyr[3][11]
 #ifdef XDIAL_C
 =
 {{
-  {12,0,0,40,17,OB_NONE,OB_NONE},
+  {11,0,0,40,16,OB_NONE,OB_NONE},
   {OB_CENTERED|OB_UNDERLINED,1,1,38,1,TOB(TX_XSKAT),OB_NONE},
   {OB_CENTERED,1,3,38,1,TOB(TX_COPYRIGHT),OB_NONE},
-  {OB_CENTERED,1,4,38,1,TOB(TX_EMAIL),OB_NONE},
-  {OB_CENTERED,1,6,38,1,TOB(TX_HOMEPAGES),OB_NONE},
-  {OB_CENTERED,1,7,38,1,TOB(TX_URL1),OB_NONE},
-  {OB_CENTERED,1,8,38,1,TOB(TX_URL2),OB_NONE},
-  {OB_CENTERED,1,10,38,1,TOB(TX_FR1),OB_NONE},
-  {OB_CENTERED,1,11,38,1,TOB(TX_FR2),OB_NONE},
-  {OB_CENTERED,1,12,38,1,TOB(TX_FR3),OB_NONE},
-  {OB_CENTERED,1,13,38,1,TOB(TX_FR4),OB_NONE},
-  {OB_EXIT,13,15,14,1,TOB(TX_WEITER),OB_NONE}
+  {OB_CENTERED,1,4,38,1,TOB(TX_URL),OB_NONE},
+  {OB_CENTERED,1,6,38,1,TOB(TX_FR1),OB_NONE},
+  {OB_CENTERED,1,7,38,1,TOB(TX_FR2),OB_NONE},
+  {OB_CENTERED,1,8,38,1,TOB(TX_FR3),OB_NONE},
+  {OB_CENTERED,1,9,38,1,TOB(TX_FR4),OB_NONE},
+  {OB_CENTERED,1,11,38,1,TOB(TX_DOWNLOAD1),OB_NONE},
+  {OB_CENTERED,1,12,38,1,TOB(TX_DOWNLOAD2),OB_NONE},
+  {OB_EXIT,13,14,14,1,TOB(TX_WEITER),OB_NONE}
 }}
 #endif
 ;
 
-EXTERN OBJECT digrafik[3][11]
+EXTERN OBJECT dicopyralt[3][11]
 #ifdef XDIAL_C
 =
 {{
-  {11,0,0,40,11,OB_NONE,OB_NONE},
+  {11,0,0,40,16,OB_NONE,OB_NONE},
+  {OB_CENTERED|OB_UNDERLINED,1,1,38,1,TOB(TX_XSKAT),OB_NONE},
+  {OB_CENTERED,1,3,38,1,TOB(TX_COPYRIGHT),OB_NONE},
+  {OB_CENTERED,1,4,38,1,TOB(TX_URL),OB_NONE},
+  {OB_CENTERED,1,6,38,1,TOB(TX_FR3),OB_NONE},
+  {OB_CENTERED,1,7,38,1,TOB(TX_FR4),OB_NONE},
+  {OB_CENTERED,1,9,38,1,TOB(TX_ALT1),OB_NONE},
+  {OB_CENTERED,1,10,38,1,TOB(TX_ALT2),OB_NONE},
+  {OB_CENTERED,1,11,38,1,TOB(TX_ALT3),OB_NONE},
+  {OB_CENTERED,1,12,38,1,TOB(TX_ALT4),OB_NONE},
+  {OB_EXIT,13,14,14,1,TOB(TX_WEITER),OB_NONE}
+}}
+#endif
+;
+
+EXTERN OBJECT digrafik[3][14]
+#ifdef XDIAL_C
+=
+{{
+  {14,0,0,40,13,OB_NONE,OB_NONE},
   {OB_CENTERED|OB_UNDERLINED,1,1,38,1,TOB(TX_GRAFIK_UND_TEXT_T),OB_NONE},
   {OB_NONE,2,3,10,1,TOB(TX_BLATT),OB_NONE},
   {OB_EXIT,2,4,23,1,TOB(TX_FRANZOESISCH),1},
@@ -660,44 +684,42 @@ EXTERN OBJECT digrafik[3][11]
   {OB_NONE,28,3,10,1,TOB(TX_SPRACHE),OB_NONE},
   {OB_EXIT,28,4,10,1,TOB(TX_GERMAN),1},
   {OB_EXIT,28,5,10,1,TOB(TX_ENGLISH),-1},
-  {OB_EXIT,13,9,14,1,TOB(TX_WEITER),OB_NONE}
+  {OB_NONE,2,9,10,1,TOB(TX_NAME),OB_NONE},
+  {OB_EXIT,10,9,11,1,OB_NONE,OB_NONE},
+  {OB_EXIT,22,9,11,1,OB_NONE,OB_NONE},
+  {OB_EXIT,13,11,14,1,TOB(TX_WEITER),OB_NONE}
 }}
 #endif
 ;
 
-EXTERN OBJECT distrateg[3][29]
+EXTERN OBJECT distrateg[3][24]
 #ifdef XDIAL_C
 =
 {{
-  {29,0,0,40,11,OB_NONE,OB_NONE},
+  {24,0,0,40,14,OB_NONE,OB_NONE},
   {OB_CENTERED|OB_UNDERLINED,1,1,38,1,TOB(TX_COMPUTER_T),OB_NONE},
-  {OB_NONE,1,3,10,1,TOB(TX_STRATEGIE),OB_NONE},
-  {OB_NONE,12,3,10,1,TOB(TX_PASSE_K),OB_NONE},
-  {OB_RIGHT,29,3,10,1,TOB(TX_SAGE18),OB_NONE},
+  {OB_NONE,1,3,20,1,TOB(TX_SPIELSTAERKE),OB_NONE},
   {OB_NONE,2,4,9,1,TOB(TX_LINKS),OB_NONE},
-  {OB_BUTTON,12,4,3,1,OB_NONE,1},
-  {OB_BUTTON,15,4,3,1,OB_NONE,1},
-  {OB_BUTTON,18,4,3,1,OB_NONE,1},
-  {OB_BUTTON,21,4,3,1,OB_NONE,1},
-  {OB_BUTTON,24,4,3,1,OB_NONE,1},
-  {OB_BUTTON,27,4,3,1,OB_NONE,1},
-  {OB_BUTTON,30,4,3,1,OB_NONE,1},
-  {OB_BUTTON,33,4,3,1,OB_NONE,1},
-  {OB_BUTTON,36,4,3,1,OB_NONE,-8},
+  {OB_BUTTON,12,4,5,1,TOB(TX_MM),1},
+  {OB_BUTTON,17,4,5,1,TOB(TX_M),1},
+  {OB_BUTTON,22,4,5,1,TOB(TX_0),1},
+  {OB_BUTTON,27,4,5,1,TOB(TX_P),1},
+  {OB_BUTTON,32,4,5,1,TOB(TX_PP),-4},
   {OB_NONE,2,5,9,1,TOB(TX_RECHTS),OB_NONE},
-  {OB_BUTTON,12,5,3,1,OB_NONE,1},
-  {OB_BUTTON,15,5,3,1,OB_NONE,1},
-  {OB_BUTTON,18,5,3,1,OB_NONE,1},
-  {OB_BUTTON,21,5,3,1,OB_NONE,1},
-  {OB_BUTTON,24,5,3,1,OB_NONE,1},
-  {OB_BUTTON,27,5,3,1,OB_NONE,1},
-  {OB_BUTTON,30,5,3,1,OB_NONE,1},
-  {OB_BUTTON,33,5,3,1,OB_NONE,1},
-  {OB_BUTTON,36,5,3,1,OB_NONE,-8},
+  {OB_BUTTON,12,5,5,1,TOB(TX_MM),1},
+  {OB_BUTTON,17,5,5,1,TOB(TX_M),1},
+  {OB_BUTTON,22,5,5,1,TOB(TX_0),1},
+  {OB_BUTTON,27,5,5,1,TOB(TX_P),1},
+  {OB_BUTTON,32,5,5,1,TOB(TX_PP),-4},
   {OB_NONE,1,7,10,1,TOB(TX_VORSCHLAEGE),OB_NONE},
   {OB_BUTTON,12,7,8,1,TOB(TX_NEIN),1},
   {OB_BUTTON,20,7,8,1,TOB(TX_JA),-1},
-  {OB_EXIT,13,9,14,1,TOB(TX_WEITER),OB_NONE}
+  {OB_NONE,1,9,10,1,TOB(TX_NAME),OB_NONE},
+  {OB_EXIT,12,9,11,1,OB_NONE,OB_NONE},
+  {OB_EXIT,26,9,11,1,OB_NONE,OB_NONE},
+  {OB_EXIT,12,10,11,1,OB_NONE,OB_NONE},
+  {OB_EXIT,26,10,11,1,OB_NONE,OB_NONE},
+  {OB_EXIT,13,12,14,1,TOB(TX_WEITER),OB_NONE}
 }}
 #endif
 ;
@@ -806,17 +828,99 @@ EXTERN OBJECT digeschwindigkeit[3][18]
 #endif
 ;
 
+EXTERN OBJECT dimehrspieler[]
+#ifdef XDIAL_C
+=
+{
+  {6,0,0,38,11,OB_NONE,OB_NONE},
+  {OB_CENTERED|OB_UNDERLINED,1,1,36,1,TOB(TX_MEHRSPIELER),OB_NONE},
+  {OB_EXIT,2,3,34,1,TOB(TX_LAN_T),OB_NONE},
+  {OB_EXIT,2,5,34,1,TOB(TX_IRC_T),OB_NONE},
+  {OB_EXIT,9,7,20,1,TOB(TX_MAN_XSKAT),OB_NONE},
+  {OB_EXIT,12,9,14,1,TOB(TX_WEITER),OB_NONE}
+}
+#endif
+;
+
+EXTERN OBJECT dilanspiel[]
+#ifdef XDIAL_C
+=
+{
+  {5,0,0,38,9,OB_NONE,OB_NONE},
+  {OB_CENTERED|OB_UNDERLINED,1,1,36,1,TOB(TX_LAN_T),OB_NONE},
+  {OB_EXIT,2,3,34,1,TOB(TX_EIGENER_TISCH),OB_NONE},
+  {OB_EXIT,2,5,34,1,TOB(TX_ANDERER_TISCH),OB_NONE},
+  {OB_EXIT,12,7,14,1,TOB(TX_ZURUECK),OB_NONE}
+}
+#endif
+;
+
+EXTERN OBJECT dieigenertisch[]
+#ifdef XDIAL_C
+=
+{
+  {14,0,0,40,15,OB_NONE,OB_NONE},
+  {OB_CENTERED|OB_UNDERLINED,1,1,38,1,TOB(TX_LAN_T),OB_NONE},
+  {OB_NONE,2,3,22,1,TOB(TX_SPIELER_2),OB_NONE},
+  {OB_BUTTON,28,3,5,1,TOB(TX_JA),1},
+  {OB_BUTTON,33,3,5,1,TOB(TX_NEIN),-1},
+  {OB_NONE,2,4,36,1,TOB(TX_RECHNER_IP),OB_NONE},
+  {OB_EXIT,2,5,36,1,OB_NONE,OB_NONE},
+  {OB_NONE,2,7,22,1,TOB(TX_SPIELER_3),OB_NONE},
+  {OB_BUTTON,28,7,5,1,TOB(TX_JA),1},
+  {OB_BUTTON,33,7,5,1,TOB(TX_NEIN),-1},
+  {OB_NONE,2,8,36,1,TOB(TX_RECHNER_IP),OB_NONE},
+  {OB_EXIT,2,9,36,1,OB_NONE,OB_NONE},
+  {OB_EXIT,7,11,26,1,TOB(TX_TISCH_OEFFNEN),OB_NONE},
+  {OB_EXIT,13,13,14,1,TOB(TX_ZURUECK),OB_NONE}
+}
+#endif
+;
+
+EXTERN OBJECT dianderertisch[]
+#ifdef XDIAL_C
+=
+{
+  {11,0,0,40,16,OB_NONE,OB_NONE},
+  {OB_CENTERED|OB_UNDERLINED,1,1,38,1,TOB(TX_LAN_T),OB_NONE},
+  {OB_NONE,2,3,22,1,TOB(TX_TISCH),OB_NONE},
+  {OB_NONE,2,4,36,1,TOB(TX_RECHNER_IP),OB_NONE},
+  {OB_EXIT,2,5,36,1,OB_NONE,OB_NONE},
+  {OB_CENTERED,2,7,36,1,TOB(TX_ZUGRIFF_1),OB_NONE},
+  {OB_CENTERED,2,8,36,1,TOB(TX_ZUGRIFF_2),OB_NONE},
+  {OB_EXIT,10,9,20,1,TOB(TX_MAN_XHOST),OB_NONE},
+  {OB_CENTERED,2,12,36,1,TOB(TX_SCHLIESST_FENSTER),OB_NONE},
+  {OB_EXIT,7,11,26,1,TOB(TX_PLATZ_NEHMEN),OB_NONE},
+  {OB_EXIT,13,14,14,1,TOB(TX_ZURUECK),OB_NONE}
+}
+#endif
+;
+
+EXTERN OBJECT diwarteauf[]
+#ifdef XDIAL_C
+=
+{
+  {5,0,0,38,8,OB_NONE,OB_NONE},
+  {OB_CENTERED|OB_UNDERLINED,1,1,36,1,TOB(TX_LAN_T),OB_NONE},
+  {OB_CENTERED,2,3,34,1,OB_NONE,OB_NONE},
+  {OB_CENTERED,2,4,34,1,OB_NONE,OB_NONE},
+  {OB_EXIT,12,6,14,1,TOB(TX_ZURUECK),OB_NONE}
+}
+#endif
+;
+
 EXTERN OBJECT diirc[]
 #ifdef XDIAL_C
 =
 {
-  {7,0,0,38,12,OB_NONE,OB_NONE},
-  {OB_CENTERED|OB_UNDERLINED,1,1,36,1,TOB(TX_IRC_T),OB_NONE},
-  {OB_CENTERED,1,3,36,1,TOB(TX_IRC_VERBINDUNG_HERSTELLEN),OB_NONE},
-  {OB_EXIT,2,5,34,1,OB_NONE,OB_NONE},
-  {OB_CENTERED,1,7,36,1,TOB(TX_IRC_KONFIGURATION_SIEHE),OB_NONE},
-  {OB_CENTERED,1,8,36,1,TOB(TX_IRC_MAN_XSKAT),OB_NONE},
-  {OB_EXIT,12,10,14,1,TOB(TX_WEITER),OB_NONE}
+  {8,0,0,40,13,OB_NONE,OB_NONE},
+  {OB_CENTERED|OB_UNDERLINED,1,1,38,1,TOB(TX_IRC_T),OB_NONE},
+  {OB_CENTERED,1,3,38,1,TOB(TX_IRC_VERBINDUNG_HERSTELLEN),OB_NONE},
+  {OB_EXIT,2,4,36,1,OB_NONE,OB_NONE},
+  {OB_EXIT,13,6,14,1,TOB(TX_START),OB_NONE},
+  {OB_CENTERED,1,8,38,1,TOB(TX_IRC_KONFIGURATION_SIEHE),OB_NONE},
+  {OB_EXIT,13,9,14,1,TOB(TX_MAN_XSKAT),OB_NONE},
+  {OB_EXIT,13,11,14,1,TOB(TX_ZURUECK),OB_NONE}
 }
 #endif
 ;
@@ -849,4 +953,17 @@ EXTERN OBJECT digui[3][20]
 #endif
 ;
 
+EXTERN OBJECT diinput[3][5]
+#ifdef XDIAL_C
+=
+{{
+  {5,0,0,40,7,OB_NONE,OB_NONE},
+  {OB_CENTERED|OB_UNDERLINED,1,1,38,1,OB_NONE,OB_NONE},
+  {OB_NONE,2,3,36,1,OB_NONE,OB_NONE},
+  {OB_EXIT,2,5,10,1,TOB(TX_LOESCHEN),OB_NONE},
+  {OB_EXIT,28,5,10,1,TOB(TX_FERTIG),OB_NONE}
+}}
+#endif
+;
+
 #endif /* XDIAL_H */
